// Linux headers
#include <fcntl.h>      // File Control wie O_RDWR
#include <errno.h>      // Error integer and strerror() function
#include <termios.h>    // Contains POSIX terminal control definitions
#include <unistd.h>     // write(), read(), close()
#include <sys/ioctl.h>
#include <string>       // std::string, std::stoi
#include <iostream>     // Allgemein fuer Streams
#include <fstream>      // Zusaetzlich fuer Dateioperationen

std::string strSio;     //Name der verwendeten Schnittstelle
struct termios tty;     //Für Schnittstellenparameter
int SerPort;            //Nummer der SIO

strSio = "/dev/ttyACM0";      // Schnittstelle ACM0 voreinstellen

//Schnittstelle suchen und konfigurieren
  SerPort = open(strSio.c_str(), O_RDWR);
  if(tcgetattr(SerPort, &tty) != 0)    //Attribute lesen
  {// printf("Fehler %i von tcgetattr: %s - %s\n", errno, strerror(errno), strSio.c_str());
    printf("Gerät nicht gefunden: %s\n", strSio.c_str());
    exit(KeinGeraet);
  }
SPort = SerPort;
  tty.c_cflag &= ~PARENB;   //Keine Parität
  tty.c_cflag &= ~CRTSCTS;  //RTS/CTS sperren
  tty.c_lflag &= ~ISIG;     //Signalzeichen INTR, QUIT und SUSP sperren
  tty.c_lflag &= ~ECHO;     //Echo sperren
  tty.c_lflag &= ~ECHOE;    //Erasure sperren
  tty.c_lflag &= ~ECHONL;   //New-line echo sperren
  tty.c_lflag &= ~ICANON;   //Kanonischer Modus (nicht Zeilenweise lesen)
  tty.c_cflag |= CREAD | CLOCAL; //Lesen ohne Steuerleitungen
  tty.c_cflag &= ~CSIZE;    //Sizebit löschen
  tty.c_cflag |= CS8;       //8 Bit je Zeichen
  tty.c_cflag &= ~CSTOPB;   //Ein Stopbit
  tty.c_lflag &= ~ISIG;     //Signalzeichen INTR, QUIT und SUSP sperren
  tty.c_iflag &= ~(IXON | IXOFF | IXANY); //s/w flow ctrl abschalten
  tty.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL); //Keine Sonderbehandlung
  tty.c_oflag &= ~OPOST;    // Prevent special interpretation of output bytes (e.g. newline chars)
  tty.c_oflag &= ~ONLCR;    // Prevent conversion of newline to carriage return/line feed
  tty.c_cc[VTIME] = 10;     // 1Sekunde Timeout (10 Dezisekunden)
  tty.c_cc[VMIN] = 0;

// Baudraten gemeinsam auf 2400 setzen
//   cfsetspeed(&tty, B2400);
   
// Tx/Rx Baudraten getrennt auf 2400 setzen
  cfsetispeed(&tty, B2400);
  cfsetospeed(&tty, B2400);

// Save tty settings, also checking for error
  if(tcsetattr(SerPort, TCSANOW, &tty) != 0)
     printf("Fehler %i von tcsetattr: %s\n", errno, strerror(errno));

// Acquire non-blocking exclusive lock
  if(flock(SerPort, LOCK_EX | LOCK_NB) == -1) {
    throw std::runtime_error("Gerät " +
          std::to_string(SerPort) + " ist bereits belegt.");
  }
//Ende - Schnittstelle suchen und konfigurieren

int16_t ReadTelex(void)
{ int16_t iNum;
  ioctl(SPort, FIONREAD, &iNum);
  if(iNum < 10)
      return(0);
  do
  { iNum = read(SPort, &cDaten, 1);
    if(cDaten[0] == 0xDC)
      break;
  } while(iNum);
  iNum = read(SPort, &cDaten[1], 9);
  if(iNum > 8)      //Falls Empfang, synchronisieren
  { for(int16_t iC = 0; iC < iNum; iC++)
    { if(cDaten[0] == 0xDC)
        if(cDaten[1] == 0xBA)
          if(cDaten[2] == 0x01)
            return(0x06);       //Return OK
      for(int16_t iZ = 0; iZ < iNum+2; iZ++)
        cDaten[iZ] = cDaten[iZ+1];  //<Puffer> linksrücken
    }
  }
  return(0);
}


